#!/usr/bin/env python
# -*- coding: utf-8 -*-
# Version
major = 0
minor = '7'
#
import os
import sys
import time
import getopt
import pickle
import random
import urllib2
import Tkinter
import webbrowser

windows = False
try:
    import _winreg
    windows = True
except:
    pass

LOCKFILE = 'wxlatest.lck'

if windows:
    # currently Image is needed for wallpaper conversion for win
    import Image
    # also ctypes to interact wallpaper gotchas.
    # you should install it separately, if python ver < 2.5
    import ctypes
    WINVER = sys.getwindowsversion()
    UA = 'WxLatest/%d.%s (Win %d.%d)' % (major, minor, 
                WINVER[0], WINVER[1])
else:
    UA = 'WxLatest/%d.%s (%s)' % (major, minor, sys.platform)

TIMEZONE = time.timezone

err = sys.stderr
RFC822tf = '%a, %d %b %Y %H:%M:%S GMT'

SUCCESS = 0
RETRY   = 1
NOAVAIL = 2

# (Label, URL, cache)
URLs = [('日本付近赤外画像',
         'http://weather.is.kochi-u.ac.jp/FE/00Latest.jpg', 'FE.jpg'),
        ('東南アジア赤外画像',
         'http://weather.is.kochi-u.ac.jp/SE/00Latest.jpg', 'SE.jpg'),
        ('ひまわり全球画像',
         'http://weather.is.kochi-u.ac.jp/QL/00Latest.jpg', 'QL.jpg'),
        ('日本付近水蒸気画像',
         'http://weather.is.kochi-u.ac.jp/WV/00Latest.jpg', 'WV.jpg'),
        ('日本付近スプリットチャンネル画像',
         'http://weather.is.kochi-u.ac.jp/SP/00Latest.jpg', 'SP.jpg'),
        ('日本付近短波長赤外差分画像',
         'http://weather.is.kochi-u.ac.jp/SD/00Latest.jpg', 'SD.jpg'),
        ('日本付近可視画像',
         'http://weather.is.kochi-u.ac.jp/JPN/00LatestX.jpg', 'JX.jpg'),
        ]
DUMMY = [('古い画像',
         'http://weather.is.kochi-u.ac.jp/JV/00Latest.jpg', 'JV.jpg'),
         ('無い画像',
          'http://weather.is.kochi-u.ac.jp/XX/00Latest.jpg', 'XX.jpg'),
         ('禁止画像',
          'http://weather.is.kochi-u.ac.jp/FV/00Latest.jpg', 'FV.jpg'),
         ]

def getproxy():
    proxy = os.getenv('http_proxy', None)
    try:
        key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER,
          'Software\\Microsoft\\Windows\\CurrentVersion\\Internet Settings')
        if _winreg.QueryValueEx(key, 'ProxyEnable')[0]:
            proxy = _winreg.QueryValueEx(key, 'ProxyServer')[0]
    except:
        pass
    return proxy

class WxLatest:
    def __init__(self, master=None):
        master.title('WxLatest')
        self.master = master
        self.menubar = Tkinter.Menu(master)
        self.makeMenu()
        master.config(menu=self.menubar)
        self.frame = Tkinter.Frame(master)
        self.makeWidgets()
        self.frame.pack()
        self.wp = 0
        self.next = None
        self.retry = 0
        self.offset = 0

    def makeMenu(self):
        global URLs
        self.imgmenu = Tkinter.Menu(self.menubar, tearoff=0)
        self.img = Tkinter.IntVar()
        self.img.set(0)
        if DEBUG: URLs = URLs + DUMMY
        for i in range(len(URLs)):
            text = URLs[i][0]
            self.imgmenu.add_radiobutton(label=text,
                                         variable=self.img, value=i)
        self.imgmenu.add_separator()
        self.imgmenu.add_command(label='終了', command=self.quit)
        self.menubar.add_cascade(label='画像選択', menu=self.imgmenu)
        #
        self.dispmenu = Tkinter.Menu(self.menubar, tearoff=0)
        self.wpx = Tkinter.IntVar()
        self.wpx.set(0)
        self.dispmenu.add_radiobutton(label='ビューアに表示',
                                      variable=self.wpx, value=0)
        if windows:
            self.dispmenu.add_radiobutton(label='壁紙に表示',
                                      variable=self.wpx, value=1)
        self.dispmenu.add_separator()
        self.dispmenu.add_command(label='気象情報頁を確認',
                                  command=self.wx_check)
        self.menubar.add_cascade(label='表示', menu=self.dispmenu)

    def makeWidgets(self):
        bx = Tkinter.Button(self.frame,
                text='更新をチェック',
                command=self.check)
        bx.pack()
        self.notice = Tkinter.StringVar()
        Tkinter.Label(self.frame, textvariable=self.notice).pack()
        self.notice.set('')
        bx = Tkinter.Button(self.frame,
                text='終了',
                command=self.quit)
        bx.pack()

    def _setvalues(self):
        i = self.img.get()
        self.URL = URLs[i][1]
        self.cache = URLs[i][2]
        self.wp = self.wpx.get()
        self.info = {}
        
    def check(self):
        self.retry = 0
        self.wx_loop()

    def wx_check(self):
        webbrowser.open('http://weather.is.kochi-u.ac.jp/')

    def fetch(self):
        if DEBUG: print >> err, 'Fetching:', self.URL
        req = urllib2.Request(self.URL)
        proxy = getproxy()
        if proxy:
            if DEBUG: print >> err, 'Using proxy:', proxy
            req.set_proxy(proxy, 'http')
        try:
            t = time.gmtime(os.stat(self.cache)[8])
            retrtime = time.strftime(RFC822tf, t)
            req.add_header('If-Modified-Since', retrtime)
        except OSError:
            pass
        req.add_header('User-Agent', UA)
        if self.retry:
            req.add_header('Pragma', 'no-cache')
        try:
            conn = urllib2.urlopen(req)
            self.info = conn.info()
            gmt = time.mktime(time.gmtime(time.time()))
            svt = time.mktime(time.strptime(self.info.get('date'), RFC822tf))
            age = self.info.get('age', '')
            if age:
                if DEBUG: print >> err, 'Proxy-cache Age:', age
                svt = svt + int(age)
            self.offset = svt - gmt
            if DEBUG: print >> err, 'TIME offset: server - client =',\
                                   self.offset, 'sec'
            fo = open(self.cache, 'wb')
            fo.write(conn.read())
            fo.close()
            conn.close()
            # set mtime equal to the server
            t = time.strptime(self.info.get('last-modified'), RFC822tf)
            t = time.mktime(t) - TIMEZONE
            os.utime(self.cache, (t, t))
            self.notice.set('Last update: ' +\
                    time.strftime('%m/%d %H:%M', time.localtime(t)))
            self.retry = 0
            return SUCCESS
        except urllib2.HTTPError, e:
            # 304 Not Modified
            if DEBUG: print >> err, 'HTTPError: ', e
            if '304' in str(e):
                ct = time.strftime('%m/%d %H:%M',
                               time.localtime(os.stat(self.cache)[8]))
                self.notice.set('Not Modified Since: %s' % ct)
                return SUCCESS
            elif '404' in str(e):
                self.notice.set('Not Found: check WxHome now!')
                return NOAVAIL
            elif '403' in str(e):
                self.notice.set('Forbidden: check WxHome now!')
                return NOAVAIL
            else:
                self.notice.set(str(e))
            return RETRY # Questionable
        except urllib2.URLError, e:
            if DEBUG: print >> err, 'URLError: ', e
            self.notice.set('Network Error: %s' % e)
            return RETRY
        return RETRY

    def wallpaper(self):
        img = Image.open(self.cache)
        img.save(self.cache + '.bmp')
        SPI_SETDESKWALLPAPER = 20
        ctypes.windll.user32.SystemParametersInfoA(
            SPI_SETDESKWALLPAPER, 0, self.cache + '.bmp' , 0)

    def wp_or_show(self):
        try:
            if self.wp:
                self.wallpaper()
            else:
                if windows:
                    os.system('start %s' % self.cache)
                elif sys.platform == 'darwin':
                    os.system('open %s' % self.cache)
                elif sys.platform == 'linux2':
                    os.system('xv %s &' % self.cache)
        except:
            pass

    def wait(self):
        x = self.info.get('last-modified')
        if x:
            if DEBUG: print >> err, 'Last-Modified:', x
            e = time.strptime(x, RFC822tf)
            e = time.mktime(e) + 3600 - TIMEZONE
        else:
            # couldn't get last-modified
            try:
                e = os.stat(self.cache)[8] + 3600
            except:
                e = 0
            if DEBUG: print >> err, 'Local Cache expires:',\
                            time.asctime(time.localtime(e))
        t = int(e - time.time() - self.offset)
        if t <= 0:
            if self.retry > 6:
                retry = 6
            else:
                retry = self.retry
            self.retry += 1
            t = 60 * (2 ** retry) # 1,2,4,8,16,32,64 min
            if DEBUG: print >> err, 'Expired: Next Try in %s sec' % t
        t = t + random.randint(20,35) # randomize in 15 sec
        if self.next:
            self.master.after_cancel(self.next)
        if DEBUG: print >> err, 'Sleeping: %s sec' % t
        self.next = self.master.after(t*1000, self.wx_loop)

    def wx_loop(self):
        self._setvalues()
        x = self.fetch()
        if x == SUCCESS:
            self.wp_or_show()
            self.wait()
        elif x == RETRY:
            try:
                os.stat(self.cache)
                self.wp_or_show()
            except:
                pass
            self.wait()
        else: # NOAVAIL
            if self.next:
                self.master.after_cancel(self.next)

    def quit(self):
        if self.next:
            self.master.after_cancel(self.next)
        self.frame.quit()


def main():
    global DEBUG
    opts, args = getopt.getopt(sys.argv[1:], 'd', ['debug'])
    DEBUG = False
    for o, a in opts:
        if o in ('-d', '--debug'):
            DEBUG = True
    # start of GUI
    root = Tkinter.Tk()
    try:
        if windows:
            root.iconbitmap('wxhome.ico')
        elif sys.platform == 'linux2':
            root.iconbitmap('@%s/wxhome.xbm' % os.getcwd())
    except:
        pass
    app = WxLatest(root)
    # get params of previous run
    try:
        p = open('wxlatest.pck', 'rb')
        appdata = pickle.load(p)
        p.close()
        app.img.set(appdata['img'])
        app.wpx.set(appdata['wpx'])
    except:
        pass
    app.wx_loop()
    root.mainloop()
    # exiting process: save params
    appdata = {'img': app.img.get(), 'wpx': app.wpx.get()}
    if DEBUG: print appdata
    p = open('wxlatest.pck', 'wb')
    pickle.dump(appdata, p)
    p.close()
    try:
        root.destroy()
    except:
        pass

def locked_main():
    try:
        os.stat(LOCKFILE)
        print >> err, 'Looks like another WxLatest is running.'
        print >> err, 'Stop it, or delete the lock file.'
        return
    except:
        open(LOCKFILE, 'w').close()
        main()
        os.unlink(LOCKFILE)

if __name__ == '__main__':
    locked_main()

